function [MotionX, MotionY] = EstimateBlockMotion(Block1, Block2)

% Block1: Bloc au temps "t"
% Block2: Bloc au temps "t+1" (Block1 dplace de "MotionX" units
% en X et "MotionY" units en Y

if (size(Block1)~=size(Block2))
   disp('EstimateBlockMotion: Les blocs ne sont pas de la mme taille!');
   disp('Erreur!!!');
	return;   
end

[N, M] = size(Block1);
if (M~=N)
   disp('EstimateBlockMotion: Les arguments doivent tre des matrices carrees!');
   disp('Erreur!!!');
   return;
end

% Correction: la rfrence (point (1,1) ou (0,0)) dans les blocs dans le cas
% de Matlab  se trouve en haut  gauche et non en bas  gauche comme dans
% l'article. Il faut donc inverser la composante en Y des blocs.
%CorrBlock1 = zeros(N,N);
%CorrBlock2 = zeros(N,N);
%for m=1:N
%   for n=1:N
%		CorrBlock1(m,n) = Block1((N-m+1),n);
%		CorrBlock2(m,n) = Block2((N-m+1),n);
%   end
%end
%Block1=CorrBlock1;
%Block2=CorrBlock2;

Block1CC = DcctI_2D(Block1);	% Equation 25
Block1CS = DcstI_2D(Block1);	% Equation 26
Block1SC = DsctI_2D(Block1);	% Equation 27
Block1SS = DsstI_2D(Block1);	% Equation 28

Block2CC = DcctII_2D(Block2);	% Equation 21
Block2CS = DcstII_2D(Block2);	% Equation 22
Block2SC = DsctII_2D(Block2);	% Equation 23
Block2SS = DsstII_2D(Block2);	% Equation 24

% Initialisation des variables
TBlock2 = zeros(4,1,(N+1),(N+1));
TBlock1 = zeros(4,4,(N+1),(N+1));
K = zeros((N+1),(N+1));
Theta = zeros(4,1,(N+1),(N+1));
C = ones((N+1),1);
Gcs = zeros((N+1),(N+1));
Gsc = zeros((N+1),(N+1));
FMuMv = zeros((N+1),(N+1));
GMuMv = zeros((N+1),(N+1));
DCS = zeros(N,N);
DSC = zeros(N,N);

% Construction du vecteur C
C(1) = 1/sqrt(2);
C(N+1) = 1/sqrt(2);

% Construction de la matrice TBlock2 (Matrice 4-D)
TBlock2(1,1,:,:) = Block2CC;		% Equation 32
TBlock2(2,1,:,:) = Block2CS;
TBlock2(3,1,:,:) = Block2SC;
TBlock2(4,1,:,:) = Block2SS;

% Construction de la matrice TBlock1 (Matrice 4-D)
TBlock1(1,1,:,:) = Block1CC;
TBlock1(1,2,:,:) = -Block1CS;
TBlock1(1,3,:,:) = -Block1SC;
TBlock1(1,4,:,:) = Block1SS;
TBlock1(2,1,:,:) = Block1CS;
TBlock1(2,2,:,:) = Block1CC;
TBlock1(2,3,:,:) = -Block1SS;
TBlock1(2,4,:,:) = -Block1SC;
TBlock1(3,1,:,:) = Block1SC;
TBlock1(3,2,:,:) = -Block1SS;
TBlock1(3,3,:,:) = Block1CC;
TBlock1(3,4,:,:) = -Block1CS;
TBlock1(4,1,:,:) = Block1SS;
TBlock1(4,2,:,:) = Block1SC;
TBlock1(4,3,:,:) = Block1CS;
TBlock1(4,4,:,:) = Block1CC;

% Construction de la matrice K (Matrice 2-D)
for k=1:(N+1)					% K de l'quation 10
   for l=1:(N+1)
      determinant = det(TBlock1(:,:,l,k));
      if (determinant == 0)
         K(l,k) = 0;
      else
         K(l,k) = 1/determinant;
      end
   end
end

% Calcul de la matrice Theta   
for k=1:(N+1)					% Equivalent de l'quation 10 en 2-D
   for l=1:(N+1)
      Theta(:,:,l,k) = K(l,k)*TBlock1(:,:,l,k)'*TBlock2(:,:,l,k);
   end
end

% Calcul des matrices Gcs et Gsc
for k=1:(N+1)
   for l=1:(N+1)
      Gcs(l,k)=Theta(2,1,l,k);
      Gsc(l,k)=Theta(3,1,l,k);
   end
end

% Calcul de la matrice FMuMv (Equation 39)
% ----------------------------------------
% Pour k et l variant de 1  N-1
FMuMv = Gcs;
% Pour k=0 et l variant de 1  N-1
k=1;
for l=2:N
   if (((Block1CC(l,k)).^2+(Block1CS(l,k)).^2)==0)
      FMuMv(l,k) = 0;
   else
      FMuMv(l,k) = (Block1CC(l,k).*Block2CS(l,k)-Block1CS(l,k).*Block2CC(l,k))/...
         ((Block1CC(l,k)).^2+(Block1CS(l,k)).^2);
   end
end
% Pour l=N et k variant de 1  N-1
l=N+1;
for k=2:N
   if (((Block1CC(l,k)).^2+(Block1SC(l,k)).^2)==0)
      FMuMv(l,k) = 0;
   else
	   FMuMv(l,k) = (Block1CC(l,k).*Block2CS(l,k)+Block1SC(l,k).*Block2SS(l,k))/...
         ((Block1CC(l,k)).^2+(Block1SC(l,k)).^2);
   end
end
% Pour k=0 et l=N
k=1;
l=N+1;
   if (Block1CC(l,k)==0)
      FMuMv(l,k) = 0;
   else
		FMuMv(l,k) = Block2CS(l,k)./Block1CC(l,k);
end

% Calcul de la matrice GMuMv (Equation 40)
% ----------------------------------------
% Pour k et l variant de 1  N-1
GMuMv = Gsc;
% Pour l=0 et k variant de 1  N-1
l=1;
for k=2:N
   if (((Block1CC(l,k)).^2+(Block1SC(l,k)).^2)==0)
      GMuMv(l,k) = 0;
   else 
	   GMuMv(l,k) = (Block1CC(l,k).*Block2SC(l,k)-Block1SC(l,k).*Block2CC(l,k))./...
         ((Block1CC(l,k)).^2+(Block1SC(l,k)).^2);
   end
end
% Pour k=N et l variant de 1  N-1
k=N+1;
for l=2:N
   if (((Block1CC(l,k)).^2+(Block1CS(l,k)).^2)==0)
      GMuMv(l,k) = 0;
   else 
	   GMuMv(l,k) = (Block1CC(l,k).*Block2SC(l,k)+Block1CS(l,k).*Block2SS(l,k))./...
         ((Block1CC(l,k)).^2+(Block1CS(l,k)).^2);
   end
end
% Pour k=N et l=0
k=N+1;
l=1;
if ((Block1CC(l,k))==0)
   GMuMv(l,k) = 0;
else
   GMuMv(l,k) = Block2SC(l,k)./Block1CC(l,k);
end

% Correction des deux matrices FMuMv et GMuMv pour les valeurs plus grandes que 1
% Voir page 953 de l'article
% Mais pratiquement, il faut prendre une valeur de seuil plus grande que 1
% On prendra le nombre de coefficients dans le bloc (i.e. (N+1)*(N+1))
for k=1:(N+1)
   for l=1:(N+1)
      if (abs(FMuMv(l,k)) > (N+1)^2)
         FMuMv(l,k) = 0;
      end
      if (abs(GMuMv(l,k)) > (N+1)^2)
         GMuMv(l,k) = 0;
      end
   end
end

% Prparation des arguments de la IDCSTII et de la IDSCTII
for k=1:(N+1)
   for l=1:(N+1)
      FMuMv(l,k) = C(k).*C(l).*FMuMv(l,k);
      GMuMv(l,k) = C(k).*C(l).*GMuMv(l,k);
   end
end

% Calcul de la IDCSTII et de la IDSCTII
DCS = IDcstII_2D(FMuMv);			% Equation 41
DSC = IDsctII_2D(GMuMv);

% ----------------------------------
% Recherche de la position des peaks
% ----------------------------------

% Recherche dans tout le bloc
% ---------------------------
%[YMaxIndexDCS, XMaxIndexDCS] = find(abs(DCS)==max(max(abs(DCS))));	% Equation 45
%[YMaxIndexDSC, XMaxIndexDSC] = find(abs(DSC)==max(max(abs(DSC))));	% Equation 46

% Recherche dans un quart du bloc
% -------------------------------
%[YMaxIndexDCS, XMaxIndexDCS] = find(abs(DCS(1:N/2,1:N/2))==max(max(abs(DCS(1:N/2,1:N/2)))));	% Equation 45
%[YMaxIndexDSC, XMaxIndexDSC] = find(abs(DSC(1:N/2,1:N/2))==max(max(abs(DSC(1:N/2,1:N/2)))));	% Equation 46

% Recherche en zig-zag dans le bloc en utilisant un seuil
% -------------------------------------------------------
% Recherche dans DCS
PreviousPeak = 0;
YMaxIndexDCS = 1;
XMaxIndexDCS = 1;
% Premier triangle de DCS
for I=1:N
   for J=1:I
      % Si la valeur du peak courant est plus grande que celle du peak prcdent plus 10%
      if (abs(DCS(I-J+1,J)) > 1.1*PreviousPeak)
         PreviousPeak = abs(DCS(I-J+1,J));
         YMaxIndexDCS = I-J+1;
         XMaxIndexDCS = J;
      end
   end
end
% Deuxime triangle du DCS
for I=2:N
   for J=I:N
      if (abs(DCS(N-J+I,J)) > 1.1*PreviousPeak)
         PreviousPeak = abs(DCS(N-J+I,J));
         YMaxIndexDCS = N-J+I;
         XMaxIndexDCS = J;
      end
   end
end

% Recherche dans DSC
PreviousPeak = 0;
YMaxIndexDSC = 1;
XMaxIndexDSC = 1;
% Premier triangle de DSC
for I=1:N
   for J=1:I
      % Si la valeur du peak courant est plus grande que celle du peak prcdent plus 10%
      if (abs(DSC(I-J+1,J)) > 1.1*PreviousPeak)
         PreviousPeak = abs(DSC(I-J+1,J));
         YMaxIndexDSC = I-J+1;
         XMaxIndexDSC = J;
      end
   end
end
% Deuxime triangle du DSC
for I=2:N
   for J=I:N
      if (abs(DSC(N-J+I,J)) > 1.1*PreviousPeak)
         PreviousPeak = abs(DSC(N-J+I,J));
         YMaxIndexDSC = N-J+I;
         XMaxIndexDSC = J;
      end
   end
end

if ((length(XMaxIndexDCS) > 1) & (length(XMaxIndexDSC) > 1))
   disp('EstimateBlockMotion: Plus d''un peak dans DCS et DSC!');
   for I=1:length(YMaxIndexDCS)
      for J=1:length(YMaxIndexDSC)
         if ((YMaxIndexDCS(I) == YMaxIndexDSC(I)) & (XMaxIndexDCS(I) == XMaxIndexDSC(I)))
            XMaxIndexTemp = XMaxIndexDCS(I);
            YMaxIndexTemp = YMaxIndexDCS(I);
         end
      end
   end
   if (length(YMaxIndexTemp) > 1)      % S'il y a plus d'un peak qui correspond dans DCS et DSC
      XMaxIndex = XMaxIndexTemp(1);    % On prend le premier peak parce qu'on suppose un mouvement lent
      YMaxIndex = YMaxIndexTemp(1);
   elseif (length(YMaxIndexTemp) == 1) % S'il n'y a qu'un peak qui correspond dans DCS et DSC
      XMaxIndex = XMaxIndexTemp;       % On prend celui-l
      YMaxIndex = YMaxIndexTemp;
   else                                % Sinon la matrice est vide et on assigne un dplacement nul
      disp('EstimateBlockMotion: Assignation d''un dplacement nul.');
      MotionX = 0;
      MotionY = 0;
      return;
   end

elseif ((length(XMaxIndexDCS) > 1) & (length(XMaxIndexDSC) == 1))
   disp('EstimateBlockMotion: DCS a plus d''un peak.');
   disp('EstimateBlockMotion: Le peak de DSC a t choisi.');
   XMaxIndex = XMaxIndexDSC;
   YMaxIndex = YMaxIndexDSC;
   
elseif ((length(XMaxIndexDCS) == 1) & (length(XMaxIndexDSC) > 1))
   disp('EstimateBlockMotion: DSC a plus d''un peak.');
   disp('EstimateBlockMotion: Le peak de DCS a t choisi.');
   XMaxIndex = XMaxIndexDCS;
   YMaxIndex = YMaxIndexDCS;
   
else
   if ([YMaxIndexDCS, XMaxIndexDCS]==[YMaxIndexDSC, XMaxIndexDSC])
      XMaxIndex = XMaxIndexDCS;
		YMaxIndex = YMaxIndexDCS;
	% Calcul du NPR (nonpeak-to-peak ratio) si les peaks ne sont pas aux mme coordonnes
	% Voir page 954 de l'article
	else											% Equation 47
      disp('EstimateBlockMotion: Les deux peaks calculs ne sont pas aux mmes coordonnes.');
      % Calcul du NPR dans tout le bloc
	   NPRDCS = abs((sum(sum(DCS)))./N^2)./abs(DCS(YMaxIndexDCS, XMaxIndexDCS));
   	NPRDSC = abs((sum(sum(DSC)))./N^2)./abs(DCS(YMaxIndexDSC, XMaxIndexDSC));
      % Calcul du NPR dans un quart du bloc
%	   NPRDCS = abs((sum(sum(DCS(1:N/2,1:N/2))))./N^2)./abs(DCS(YMaxIndexDCS, XMaxIndexDCS));
%   	NPRDSC = abs((sum(sum(DSC(1:N/2,1:N/2))))./N^2)./abs(DCS(YMaxIndexDSC, XMaxIndexDSC));
	   if (NPRDSC < NPRDCS)
   		disp('EstimateBlockMotion: NPR plus petit pour DSC, donc DSC choisi.');
			XMaxIndex = XMaxIndexDSC;
	      YMaxIndex = YMaxIndexDSC;
		elseif (NPRDCS < NPRDSC)
			disp('EstimateBlockMotion: NPR plus petit pour DCS, donc DCS choisi.');
		   XMaxIndex = XMaxIndexDCS;
	      YMaxIndex = YMaxIndexDCS;
	   else
			disp('EstimateBlockMotion: NPRs identiques.');	
	      if (abs(DCS(YMaxIndexDCS, XMaxIndexDCS)) > abs(DSC(YMaxIndexDSC, XMaxIndexDSC)))
				disp('EstimateBlockMotion: Peak de DCS plus grand donc choisi.');
            XMaxIndex = XMaxIndexDCS;
	         YMaxIndex = YMaxIndexDCS;
	      else
            disp('EstimateBlockMotion: Peak de DSC plus grand donc choisi.');
			   XMaxIndex = XMaxIndexDSC;
	   	   YMaxIndex = YMaxIndexDSC;
	      end
	   end
	end
end

% Calcul du dplacement
% Direction: Nord-Est
if ((DSC(YMaxIndex,XMaxIndex) > 0) & (DCS(YMaxIndex,XMaxIndex) > 0))
   MotionX = XMaxIndex-1;
   MotionY = YMaxIndex-1;
% Direction: Sud-Est
elseif ((DSC(YMaxIndex,XMaxIndex) > 0) & (DCS(YMaxIndex,XMaxIndex) < 0))
   MotionX = XMaxIndex-1;
   MotionY = -((YMaxIndex-1)+1);
% Direction: Nord-Ouest
elseif ((DSC(YMaxIndex,XMaxIndex) < 0) & (DCS(YMaxIndex,XMaxIndex) > 0))
   MotionX = -((XMaxIndex-1)+1);
   MotionY = YMaxIndex-1;
% Direction: Sud-Ouest
elseif ((DSC(YMaxIndex,XMaxIndex) < 0) & (DCS(YMaxIndex,XMaxIndex) < 0))
   MotionX = -((XMaxIndex-1)+1);
   MotionY = -((YMaxIndex-1)+1);
% Aucun mouvement   
else
   MotionX = 0;
   MotionY = 0;
end
