%--------------------------------------------------------------
% description : Fonction permettant de reconstruire
% une image en fonction des vecteurs d'estimation de mouvement
% par rapport a l'image precedante
% auteur : Francis Pieraut
% Derniere modification: 2 avril 2001
%---------------------------------------------------------------
function [image_reconstruite,pourcentage_erreur]=Reconstruction(size,vectx,vecty,image_precedante,display)
erreur=0;
[sizey,sizex]=size(image_precedante);
% calcul de stepx et stepy
[y,x]=size(vectx);

% initialisation de l'image reconstruite
image_reconstruite= zeros(size*y,size*x);

% Reconstruction de l'image
for L = 0:(y-1),
   for C = 0:(x-1),
      dep_x = vectx(L+1,C+1);
      dep_y = vecty(L+1,C+1);
      % calcul des positions du bloc choisi dans l'image reconstruite
      CY1=L*size+1;
      CY2=L*size+size;
      CX1=C*size+1;
      CX2=C*size+size;
      % calcul des positions du bloc choisi dans l'image precedante
      Y1=CY1+dep_y;
      Y2=Y1+size-1;
      X1=CX1+dep_x;
      X2=X1+size-1;
      
      % verification que le le block idal est a l'intrieur de l'image prcdant
      if ( (Y1 >0) & (Y2<=y*size) ...
         & (X1 >0) & (X2<=x*size) )
              image_reconstruite(CY1:CY2,CX1:CX2)=...
                  image_precedante(Y1:Y2,X1:X2);
      else
          % copie de la rgion precedente
          image_reconstruite(CY1:CY2,CX1:CX2)= ...
               image_precedante(CY1:CY2,CX1:CX2);
          erreur = erreur+1;
          % effectuer cette optimisation seulement si la grandeur de l'image est infrieure  24X24
          if (x*y<=576)
            % Modification des positions du bloc choisis 
            % pour quelles ne soient pas  l'exterieur
            if (Y1<0)
              Y1=1;
            else (Y2>y*size)
              Y2=y*size;
            end;
            if (X1<0)
              X1=1;
            else (X2>x*size)
              X2=x*size;
            end;
     
            CY1=Y1-dep_y;
            CY2=Y2-dep_y;
            CX1=X1-dep_x;
            CX2=X2-dep_x;
            % derniere verification
            if ( (CY1 >0) & (CY2<=y*size) ...
                   & (CX1 >0) & (CX2<=x*size) )
                image_reconstruite(CY1:CY2,CX1:CX2)= ...
                     image_precedante(Y1:Y2,X1:X2) ;
             end;
            disp(['Reconstruction out of matrix boundary : (',num2str(L+1),',',num2str(C+1),')']);
           end;
      end;%else
	end;% for 2
end;% for 1
% affichage des resultats
if display==1
    disp('Affichage de l''image reconstruite');
    %image_precedante
    %image_reconstruite
    %imagesc(image_precedante)
    imagesc(image_reconstruite);
end
pourcentage_erreur = (erreur/(x*y))*100;




	

 














