function [vx,vy] = mpg_blockmatch_2D( IA, IB, mbSize, SearchArea, fDisplay );
% block_match perform block matching:
% requires
% - IA: first_image
% - IB: last_image
% - SearchArea : +/- Pixel Area in 2D where to look for the block in the other image
% - fDisplay: Display step by step graphical info
% - mbSize: macroblock size
%
% Laurent Pieraut, 10-july-99
%
% 31-mar-01:    LP - remove block_match_min_contrast
%               LP - remove downscaled algo
%               LP - add SearchArea parameter
%               LP - add fDisplay
%               LP - removed global variables
%

%global mbSize sizeX sizeY maxStepX maxStepY;

[sizeY, sizeX] = size(IA);
maxStepX = sizeX / mbSize;
maxStepY = sizeY / mbSize;

% display options (for debug)
disp_choice_match = 0;
disp_disp = 0;

% main variables
%bSize = mbSize/2; % block size

% quadrillage de l'image en 16x16
for j=1:maxStepY
   for i=1:maxStepX
            
      % disp(['i=' num2str(i) ', j=' num2str(j)]);
      %if (j==30)
      %    pause;
      %end
       
      % block 32 by 32 coordinates
      xmin32 = max( [1 (i-1)*mbSize - SearchArea+1 ] );
      xmax32 = min( [sizeX i*mbSize + SearchArea ]);
      ymin32 = max( [1 (j-1)*mbSize - SearchArea+1 ] );
      ymax32 = min( [sizeY j*mbSize + SearchArea ]);
      
      % block 16 by 16 coordinates
      xmin16 = (i-1)*mbSize+1;
      xmax16 = i*mbSize;
      ymin16 = (j-1)*mbSize+1;
      ymax16 = j*mbSize;
      
      % copy block
      % first image
      bk16x16 = IA( ymin16:ymax16, xmin16:xmax16);
      % last image
      bk32x32 = IB( ymin32:ymax32, xmin32:xmax32);
      
 	  % start block matching 
      mad = 1e12; % big value
 	  [bk_sizeY bk_sizeX] = size(bk32x32);
      
 	  for y=1:bk_sizeY-mbSize+1
   	    for x=1:bk_sizeX-mbSize+1
      	    A = bk32x32(y:y+mbSize-1,x:x+mbSize-1);
         	madtemp = abs( (1/(mbSize^2))*sum(sum(abs(A - bk16x16))) ) ;
            if ( mad > madtemp )
	            mad = madtemp;
   	            [vx(j,i), vy(j,i)] = mpg_displacement(x, y, SearchArea, i, j);
                A_best = A;
         	end;
            if (fDisplay)
                figure(1);
                % compute current block displacement
                [dx_tmp, dy_tmp] = mpg_displacement(x, y, SearchArea, i, j);
                % show 4 corners of block in search
                bk_temp = bk32x32;
                bk_temp(y,x) = 2;
                bk_temp(y+mbSize-1,x) = 2;
                bk_temp(y,x+mbSize-1) = 2;
                bk_temp(y+mbSize-1,x+mbSize-1) = 2;
                subplot(2,3,1); imagesc(bk16x16); title(['Block to find (Im1) (i=' num2str(i) ', j=' num2str(j) ')']);
                subplot(2,3,2); imagesc(bk_temp); title('Total Area to analyse (Im2)');
                subplot(2,3,3); imagesc(IB( ymin16:ymax16, xmin16:xmax16) ); title('Central Block (Im2)');
                subplot(2,3,4); imagesc(A_best); title(['Best Match (vx=' num2str(vx(j,i)) ', vy=' num2str(vy(j,i)) ')']);
                subplot(2,3,5); imagesc(A); title(['Current Block (vx=' num2str(dx_tmp) ', vy=' num2str(dy_tmp) ')']);
                pause;
            end;
	     end
      end
         

           
      if (disp_disp)
         %keyboard;
         disp([ 'Block(' num2str(i) ',' num2str(j) '), vx = ' num2str( vx(j,i) ) ', vy = ' num2str( vx(j,i) ) ]);
      end;
      
   end
end
