function Irb = mpg_imreconstr( Iback, Iforw, BFM, vxB, vyB, vxF, vyF, mbSize );
% mpg_imreconstr( Iback, Iforw, BFM, vx, vy ))
% mpg_imreconstr reconstruct the image based on the flg.
% Each block of the new image = corresponding block of Iback/Iforw/Iaver with (vx,vy) pixel correction
% The algorithm is based on the :
% - Iback: Previous image
% - Iforw: Next image
% - BFM: macroblock flag indicating which algo to use (Backward/Forward/Average)
% - vx : array (1:maxStepX,1:maxStepY) of displacement in X of mbSize macroblocks
%			vx represents the X displacement of the backward image block(i,j) 
%			relative to image Iback/Iforw block
% - vy : array (1:maxStepX,1:maxStepY) of displacement in Y of mbSize macroblocks
%			vy represents the Y displacement of the backward image block(i,j) 
%			relative to image Iback/Iforw block
% Laurent Pieraut, 10-july-99
%
% 31-mar-01:    LP - removed global variables

%global mbSize sizeX sizeY maxStepX maxStepY;
[sizeY, sizeX] = size(Iback);
maxStepX = sizeX / mbSize;
maxStepY = sizeY / mbSize;

% display flag
f_disp = 0;

% initialize image to matrix of zeros
Irb = zeros(size(Iback));

% reconstruction of image
for j=1:maxStepY
   for i=1:maxStepX
      % block is 0 if BFM=0 
            
      % compute backward algo block
      if (BFM(j,i) == 1)
         if (f_disp) 
            disp(['j = ' num2str(j) ', i = ' num2str(i)]);
         end
                 
      	% block 16 by 16 coordinates
      	xmin16_1 = (i-1)*mbSize+1;
      	xmax16_1 = i*mbSize;
      	ymin16_1 = (j-1)*mbSize+1;
      	ymax16_1 = j*mbSize;
            
      	xmin16_2 = xmin16_1 + vxB(j,i);
      	if (xmin16_2 < 1)
      	   xmax16_1 = xmax16_1 + xmin16_2 - 1;
      	   xmin16_2 = 1;
      	end;
      	xmax16_2 = xmax16_1 + vxB(j,i);
      	if (xmax16_2 > sizeX)
      	   xmin16_1 = xmin16_1 + xmax16_2 - sizeX;
      	   xmax16_2 = sizeX;
      	end;
      	ymin16_2 = ymin16_1 + vyB(j,i);
      	if (ymin16_2 < 1)
      	   ymax16_1 = ymax16_1 + ymin16_2 - 1;
      	   ymin16_2 = 1;
      	end;
      	ymax16_2 = ymax16_1 + vyB(j,i);
      	if (ymax16_2 > sizeY)
      	   ymin16_1 = ymin16_1 + ymax16_2 - sizeY;
         	ymax16_2 = sizeY;
         end;
         
         if (f_disp)
	         disp(['Vx = ' num2str(vxB(j,i)) ', Vy = ' num2str(vyB(j,i))]);
   		   disp(['1 - xmin = ' num2str(xmin16_1) ', xmax = ' num2str(xmax16_1) ...
         	   ', ymin = ' num2str(ymin16_1) ', ymax = ' num2str(ymax16_1)]);
         	disp(['2 - xmin = ' num2str(xmin16_2) ', xmax = ' num2str(xmax16_2) ...
                  ', ymin = ' num2str(ymin16_2) ', ymax = ' num2str(ymax16_2)]);
         end;
         
         % copy backward image
         Irb(ymin16_1:ymax16_1,xmin16_1:xmax16_1) = ...
            Iback(ymin16_2:ymax16_2,xmin16_2:xmax16_2) ;
      end;
      
      % compute backward algo block
      if (BFM(j,i) == 2)
      	% block 16 by 16 coordinates
      	xmin16_1 = (i-1)*mbSize+1;
      	xmax16_1 = i*mbSize;
      	ymin16_1 = (j-1)*mbSize+1;
      	ymax16_1 = j*mbSize;
            
      	xmin16_2 = xmin16_1 + vxF(j,i);
      	if (xmin16_2 < 1)
      	   xmax16_1 = xmax16_1 + xmin16_2 - 1;
      	   xmin16_2 = 1;
      	end;
      	xmax16_2 = xmax16_1 + vxF(j,i);
      	if (xmax16_2 > sizeX)
      	   xmin16_1 = xmin16_1 + xmax16_2 - sizeX;
      	   xmax16_2 = sizeX;
      	end;
      	ymin16_2 = ymin16_1 + vyF(j,i);
      	if (ymin16_2 < 1)
      	   ymax16_1 = ymax16_1 + ymin16_2 - 1;
      	   ymin16_2 = 1;
      	end;
      	ymax16_2 = ymax16_1 + vyF(j,i);
      	if (ymax16_2 > sizeY)
      	   ymin16_1 = ymin16_1 + ymax16_2 - sizeY;
         	ymax16_2 = sizeY;
      	end;
         % copy forward image
         Irb(ymin16_1:ymax16_1,xmin16_1:xmax16_1) = ...
            Iforw(ymin16_2:ymax16_2,xmin16_2:xmax16_2) ;
      end;
      
      % compute mean algo block
      if (BFM(j,i) == 3)
      	% block 16 by 16 coordinates
      	xmin16 = (i-1)*mbSize+1;
      	xmax16 = i*mbSize;
      	ymin16 = (j-1)*mbSize+1;
      	ymax16 = j*mbSize;
            
         % copy mean image
         Irb(ymin16:ymax16,xmin16:xmax16) = ...
            ( Iback(ymin16:ymax16,xmin16:xmax16) + ...
              Iforw(ymin16:ymax16,xmin16:xmax16) ) / 2 ;
		end;
        
	end;
end
