function comparaison(test_id,image1,image2,blocksize,sousblocksize,image_filename,...
                     seq_nb,filtre,test_boucle,display)
%-------------------------------------------------------------------------------------
% Description: Script permettant de faire la comparaison 
% entre l'estimation de mouvement dans le domaine spatial
% et le domaine frequentiel.
% auteur : Francis Pieraut 
% derniere mod: 11 avril 2001
% caracteristique: 5 types de test (TEST_ID=1,2..5 ou X)
% default (X) -> image predeterminee (voir build_image_test2)
% 1 -> test sur une serie d'images compltes
% 2 -> subdivision d'une image en plusieurs bloc  comparer
%-------------------------------------------------------------------------------------

%-----------------Asignation des paramtres-------------------------------------------
% Identificateur du test
% 0=test avec images gnres
% 1=test avec serie d'images
% 2=test avec une image relles coupes en blocks
% 3=deplacement vers l'exterieur
% 4=test avec image gnres inverses
% 5=test sur image prealablement dfinies (image1,image2)
TEST_ID = test_id;
GENEREE=0;
INV_GENEREE=1;
COUPEE=2;
EXTERIEUR=3;
NON_GENEREE=4;
SERIE=5;
SERIE_DIFF=6;
SERIE_VAR3=7; % serie avec sequence spare de 3
SERIE_DIFF_VAR3=8;
% Affichage des rsulats sous forme graphique dans une fenetre
DISPLAY_RESULTS =display;
% Grandeur des blocks pour l'estimation de mouvemement
mbSize=blocksize;
% Permet de spcifier la grandeur des sous blocks de l'images  traiter
% parametres seulement utile pour (TEST_ID==2)
sous_block_size=sousblocksize;
% Permet de vrifier que les boucles de dcoupage de block d'images fonctionnent bien
% utile dans TEST_ID = 2
TEST_BOUCLE_SEULEMENT = test_boucle;
% Permet d'activer le filtre avant de faire l'EM-DF
FILTRE=filtre;
%-----------------Parametres utiles pour le degug-------------------------------------
% parametres (0 ou 1, Actif ou Inactif)

% permet de verifier le correspondance entre Reconstruction() et mpg_imreconstr()
VERIFICATION_RECONSTRUCTION =0 ; 
% Permet d'afficher la difference entre l'estimation du mouvement et le meilleur choix
% utile dans TEST_ID = 0
AFFICHAGE_DIFF_MOTION_ATTENDU = 0;
% Permet d'indiquer l'iteration courante
DISPLAY_ITERATION = 1;
%---------------Initialisation par defaut de variables----------------------------------
% nb d'images  traiter
NB=0;
% Intialisation MotionEstimation Domaine Spatial
InitME_DS
% Initialisation des variables 
x=0;
y=0;
sans_fitre='SansFitre';
% Initialisation de la boucle en fonction de TEST_TYPE
if TEST_ID==SERIE 
     NB=seq_nb-1;
     begin_filename='TestSerie_';
elseif TEST_ID==SERIE_DIFF
    NB=seq_nb-2;
    begin_filename='TestSerieDiff_';
elseif TEST_ID==SERIE_VAR3 
    NB=seq_nb-3;
    begin_filename='TestSerieVar3_';
elseif TEST_ID==SERIE_DIFF_VAR3
    NB=seq_nb-3;
    begin_filename='TestSerieDiffVar3_';
elseif TEST_ID==COUPEE
    begin_filename='TestCoupee_';
    ImSizeX = sous_block_size;
    ImSizeY = sous_block_size;
    size= sous_block_size;
    filename1= [image_filename,'001.bmp'];
    filename2= [image_filename,'002.bmp'];
    TOTimage1 = double(imread(filename1,'bmp'));
    TOTimage2 = double(imread(filename2,'bmp'));
    [Y,X]= size(TOTimage1);
    [y1,x1]=size(TOTimage1);
    [y2,x2]=size(TOTimage2);
    if ( (y1 ~= y2) & (x1 ~=x2) )
    %if ( size(TOTimage1)~=size(TOTimage2) )
        disp('Erreur: Les images ne sont pas de la mme taille!');
        return;
    end
    [ImageSizeY,ImageSizeX] = size(TOTimage1);
    NbrBlockY = ImageSizeY/sous_block_size;
    NbrBlockX = ImageSizeX/sous_block_size;
    NbrBlocks = NbrBlockX * NbrBlockY;
    NB=NbrBlocks-1;
elseif TEST_ID==NON_GENEREE;
    begin_filename='TestNonGen_';
    NB=1;
else  %
    TEST_ID==GENEREE;
    begin_filename='TestGen_';
    NB=1;
end;
if (filtre~=1)
    disp('sans filtre');
    begin_filename=[begin_filename,sans_fitre];
else
    disp('avec filtre');
end;
% initialisation des vecteurs contenant les taux d'erreur
P_error=zeros(1,NB); %par rappport  l'image prcdante
P_error_MEDS=zeros(1,NB);
P_error_MEDF=zeros(1,NB);
pourcentage_MEDF=zeros(1,NB);
%---------------Debut du le comparaison-------------------------------------------------
% open output filename to write results
if TEST_BOUCLE_SEULEMENT==0
    output_filename=[begin_filename image_filename '.sim'];
    file=fopen(output_filename,'wt');
    disp(['Ouverture du fichier : ',output_filename]);
end;
if DISPLAY_RESULTS==1
    f=figure;
end;
if (TEST_ID==SERIE | TEST_ID==SERIE_DIFF | TEST_ID==SERIE_VAR3 | TEST_ID==SERIE_DIFF_VAR3)
        if TEST_BOUCLE_SEULEMENT==0
            fprintf(file,'serie d''image teste avec %i sequences. Blocksize= %i\n\n',NB,blocksize);
        end;
end;
% pour toutes les images
for i=1:NB
    if DISPLAY_ITERATION==1
        disp(['Image : ',num2str(i)]);
    end;
    % load images
    if (TEST_ID==SERIE | TEST_ID==SERIE_DIFF | TEST_ID==SERIE_VAR3 | TEST_ID==SERIE_DIFF_VAR3)
        filtre= [1/12 1/12 1/12;1/12 1/4 1/12;1/12 1/12 1/12];
        filtre1= [1/12 1/12 1/12;1/12 1 1/12;1/12 1/12 1/12];
        filtre5 = [0 1 0;1 -40 1;0 1 0];
        if i>=100
            filename1= [image_filename num2str(i) '.bmp'];
        elseif i>=10
            filename1= [image_filename '0' num2str(i) '.bmp'];
        else
            filename1= [image_filename '00' num2str(i) '.bmp'];
        end;
        image1 = double(imread(filename1,'bmp'));
        if (TEST_ID==SERIE_VAR3 | TEST_ID==SERIE_DIFF_VAR3)
            i=i+3;
        end;
        if (i+1)>=100
            filename2= [image_filename num2str(i+1) '.bmp'];
        elseif (i+1)>=10
            filename2= [image_filename '0' num2str(i+1) '.bmp'];
        else
            filename2= [image_filename '00' num2str(i+1) '.bmp'];
        end;
        image2 = double(imread(filename2,'bmp'));
        if (TEST_ID==SERIE_DIFF| TEST_ID==SERIE_DIFF_VAR3)
            if (i+2)>=100
                filename3= [image_filename num2str(i+2) '.bmp'];
            elseif (i+2)>=10
                filename3= [image_filename '0' num2str(i+2) '.bmp'];
            else
                filename3= [image_filename '00' num2str(i+2) '.bmp'];
            end;
            image3 = double(imread(filename3,'bmp'));
            image1=image2-image1;   
            image2=image3-image2;
        end;
        if (TEST_ID==SERIE_VAR3 | TEST_ID==SERIE_DIFF_VAR3)
            i=i-3;
        end;
        max_intensite =255;
    elseif TEST_ID==COUPEE
        disp(['decoupage : ',num2str(i),' (',num2str(x+1),',',num2str(y+1),')']); 
        image1=TOTimage1(...
            y*size+1:y*size+size,x*size+1:x*size+size);
        image2= TOTimage2(...
            y*size+1:y*size+size,x*size+1:x*size+size);
        if (x+1)<=((X / sous_block_size)-1)
            x=x+1;
        else 
            if (y+1)<=((Y/sous_block_size)-1)
                y=y+1;
                x=0;
            else
                disp('Fin du dcoupage de l''image en block'); 
                return;
            end
        end;
        max_intensite=255;
        DISPLAY_ITERATION = 0;
   elseif TEST_ID==NON_GENEREE
        max_intensite=1;
    else %GENEREE
        % Construction des 2 images prdfinies
        [image1,image2]=build_image_test_simple(0);
        max_intensite=1;
    end;
    [ImSizeY,ImSizeX] = size(image1);
    if TEST_BOUCLE_SEULEMENT == 0
        
        % Initialisation Step X and Y 
        maxStepX = ImSizeX / mbSize;
        maxStepY = ImSizeY / mbSize;
        % calcul de la moyenne de l'intensite de l'image a reconstruire
        moyenne_image2=sum(sum(abs(image2)))/(ImSizeX*ImSizeY); 
    
        % DS-ME
        disp('DS-ME');
        [Vx, Vy] = mpg_blockmatch_2D...
                  (image2,image1,mbSize,6,0);
        %save vx4 Vx
        %save vy4 Vy
        VecZeros = ones(maxStepY,maxStepX);
        RBFM = VecZeros;
        % rebuild image based on I and motion
        
        imMEDS_1 = Reconstruction(mbSize,Vx,Vy,image1,0); 
        %mpg_imreconstr(image1, [], RBFM, ... 
         %      Vx, Vy, [], [],mbSize );
        if VERIFICATION_RECONSTRUCTION == 1
            [imMEDS_2,pourcentage_MEDF(1,i)]=Reconstruction(mbSize,Vx,Vy,image1,0);
            if (sum(sum(imMEDS_1-imMEDS_2)))==0
                disp('Reconstruction=mpg_imreconstr OK');
            else
                disp('ERROR Reconstruction!=mpg_imreconstr !!!!!');
            end;
        end;
        %compute diff between reconstr image and real image
        ImDiff_MEDS =image2 - imMEDS_1;
        SumDiff= sum(sum(abs(ImDiff_MEDS)));
        MoyenDiffPixel=SumDiff./(ImSizeX*ImSizeY);
        P_error_MEDS(1,i)= MoyenDiffPixel*100/max_intensite;
        disp([num2str(i),'-ME-DS)Moyenne de difference par pixel: ',num2str(P_error_MEDS(1,i)),'%']);
    
    
        % MotionEstimation Domaine Spatial
        disp('DF-ME');
        % application du filtre
        if (FILTRE==1)
            %image1_filtree = filter2(filtre, image1);
            %image2_filtree = filter2(filtre, image2);
            image1_filtree = filter2(filtre5, image1);
            image2_filtree = filter2(filtre5, image2);
            image1_filtree = filter2(filtre1, image1_filtree);
            image2_filtree = filter2(filtre1, image2_filtree);
            [MotionX, MotionY] =EstimateMotion(image1_filtree,image2_filtree,mbSize,TEST_BOUCLE_SEULEMENT);
        else
            [MotionX, MotionY] =EstimateMotion(image1,image2,mbSize,TEST_BOUCLE_SEULEMENT);
        end;
        MotionX=-MotionX;
        MotionY=-MotionY;
        
        % Affichage de la difference par rapport aux deplacements attendus
        if AFFICHAGE_DIFF_MOTION_ATTENDU==1
            MotionAttenduX
            MotionAttenduY
            diffX=MotionX-MotionAttenduX 
            diffY=MotionY-MotionAttenduY
        end;

        % Reconstruction DF-ME
        [imMEDF_1,pourcentage_MEDF(1,i)]=Reconstruction(mbSize,MotionX,MotionY,image1,0);
        if VERIFICATION_RECONSTRUCTION == 1
            imMEDF_2 = mpg_imreconstr(image1, [], RBFM, ...
                           MotionX, MotionY, [], [],mbSize );
            ImDiff = image2 - imtmp2;
            if (sum(sum(imMEDF_1-imMEDF_2)))==0
                disp('Reconstruction=mpg_imreconstr OK');
            else
                disp('ERROR Reconstruction!=mpg_imreconstr !!!!!');
            end;
        end;
        %compute diff between reconstr image and real image
        ImDiff_MEDF = image2 - imMEDF_1;
        SumDiff= sum(sum(abs(ImDiff_MEDF))); 
        MoyenDiffPixel=SumDiff/(ImSizeX*ImSizeY);
        P_error_MEDF(1,i)= MoyenDiffPixel*100/max_intensite;
        ImDiff = image2 - image1;
        SumDiff= sum(sum(abs(ImDiff))); 
        MoyenDiffPixel=SumDiff/(ImSizeX*ImSizeY);
        P_error(1,i)= MoyenDiffPixel*100/max_intensite;
        disp([num2str(i),'-ME-DF)Moyenne de difference par pixel: '...
                ,num2str(P_error_MEDF(1,i)),'% rejets: ',num2str(pourcentage_MEDF(1,i)),...
                '% difference :',num2str(P_error_MEDF(1,i)-P_error_MEDS(1,i)),' difference image precedante : ',num2str(P_error(1,i))]);

        if DISPLAY_RESULTS==1
            if (NB<=20)  % affichage des resultats avec les images 
                figure(f);
                %suptitle(image_filename);
                % display reel image (i-1)
                subplot(NB,6,6*(i-1)+1);
                imagesc(image1);
                colormap(gray);
                axis off;
                title(['Image (i-1)',num2str(P_error(1,i)),'%']);
                % display reel image (i)
                subplot(NB,6,6*(i-1)+2);
                imagesc(image2);
                colormap(gray);
                axis off;
                title('Image i');
                % reconstruction with ME_DS 
                subplot(NB,6,6*(i-1)+3);
                imagesc(imMEDS_1);
                colormap(gray);
                axis off;
                title(['DS: ',num2str(P_error_MEDS(1,i)),'%']);
                % display difference
                subplot(NB,6,6*(i-1)+4);
                imagesc(ImDiff_MEDS);
                colormap(gray);
                axis off;
                %quiver(Vx, Vy);
                %axis equal;
                %title('vecteurs DS');
                title('Difference DS');
                % reconstruction with ME_DF
                subplot(NB,6,6*(i-1)+5);
                imagesc(imMEDF_1);
                colormap(gray);
                axis off;
                title(['DF: ',num2str(P_error_MEDF(1,i)),'%']);%,'% rejets: ',num2str(pourcentage_MEDF(1,i)),'%']);
                % display difference
                %subplot(NB,6,6*(i-1)+6);
                %quiver(MotionX, MotionY);
                %axis equal;
                %title('vecteurs DF');
                % display difference
                subplot(NB,6,6*(i-1)+6);
                imagesc(ImDiff_MEDF);
                colormap(gray);
                axis off;
                title('Difference DF');
            end
                
        end;
        fprintf(file,'erreur DS: %f %% Erreur DF: %f %% rejets: %f %% diff : %f %% diff(aucun dep): %f %%\n'...
                ,P_error_MEDS(1,i),P_error_MEDF(1,i),pourcentage_MEDF(1,i),P_error_MEDF(1,i)-P_error_MEDS(1,i),P_error(1,i));
   end;
end;
% save 
if DISPLAY_RESULTS==1
    saveas(f,[begin_filename image_filename '_seq.jpg'])
end;
% synthese des resultats
tot_P_error=sum(P_error);
tot_P_error_MEDS=sum(P_error_MEDS);
tot_P_error_MEDF=sum(P_error_MEDF);
tot_pourcentage_MEDF=sum(pourcentage_MEDF);
variation_error=abs((P_error_MEDF-P_error_MEDS)./P_error_MEDS);
tot_variation_error=sum(variation_error);


if (TEST_BOUCLE_SEULEMENT==0)
    if (NB>1)
        fprintf(file,'\n\nMOYENNE \nerror DS: %f %% Erreur DF: %f %% rejets: %f %% diff(aucun dep): %f %%\n'...
            ,(tot_P_error_MEDS/NB),(tot_P_error_MEDF/NB),(tot_pourcentage_MEDF/NB),(tot_P_error/NB));  
    end;
    %fermeture du fichier de resultat
    fclose(file);
    disp(['Fermeture du fichier : ',output_filename]);

    if (test_id==NON_GENEREE)
        f=figure;
        % suptitle(image_filename);
        % display reel image (i)
        subplot(NB,4,4*(i-1)+1);
        imagesc(image2);
        title('Image i');
        % display reel image (i-1)
        subplot(NB,4,4*(i-1)+2);
        imagesc(image1);
        title(['Image (i-1)',num2str(P_error(1,i)),'%']);
        % reconstruction with ME_DS 
        subplot(NB,4,4*(i-1)+3);
        imagesc(imMEDS_1);
        title(['DS: ',num2str(P_error_MEDS(1,i)),'%']);
        % reconstruction with ME_DF
        subplot(NB,4,4*(i-1)+4);
        imagesc(imMEDF_1);
        title(['DF: ',num2str(P_error_MEDF(1,i)),'%']);%,'% rejets: ',num2str(pourcentage_MEDF(1,i)),'%']);
        saveas(f,[begin_filename image_filename '_im.jpg'])
    else   
        if NB>1
        % affichage des % d'erreurs sous forme graphique
        disp('Affichage des % d''erreur sous forme graphique');
        f=figure;
        %suptitle(image_filename);
        subplot(1,4,1);
        plot(P_error);
        title(['Image precedante: ',num2str(tot_P_error/NB),'%']);
        % display error DS
        subplot(1,4,2);
        plot(P_error_MEDS);
        title(['DS: ',num2str(tot_P_error_MEDS/NB),'%']);
        % display error DF
        subplot(1,4,3);
        plot(P_error_MEDF);
        title(['DF: ',num2str(tot_P_error_MEDF/NB),'%']);
        % display varialtion DS DF
        subplot(1,4,4);
        plot(variation_error);
        title(['Var. erreur: ',num2str(tot_variation_error/NB),'%']);
        saveas(f,[begin_filename image_filename '_error.jpg'])
        end;
    end;
end;



